@@ -12,7 +12,7 @@ from send2trash import send2trash
 import copy
 import dpath.util
 
-# region globals
+NO_ARGUMENT = object()
 
 def create_nested_directory(path: str):
     """
@@ -33,8 +33,31 @@ def freeze(o):
 
     return hash(o)
 
-#endregion
-# region exceptions
+def convert_to_notify_structure(data: Union[dict, list], parent: Resource) -> Union[NotifyDict, NotifyList]:
+    """
+    Converts a dict or list to a notify structure.
+    """
+    if isinstance(data, dict):
+        return NotifyDict(data, owner=parent)
+
+    if isinstance(data, list):
+        return NotifyList(data, owner=parent)
+
+    return data
+
+def smart_compare(a, b):
+    """
+    Compares to objects using ==, but if they can both be interpreted as
+    path-like objects, it uses path comparison.
+    """
+    try:
+        a = Path(a)
+        b = Path(b)
+
+        return a == b
+    except Exception:
+        return a == b
+
 class ReticulatorException(Exception):
     """
     Base class for Reticulator exceptions.
@@ -56,10 +79,6 @@ class AmbiguousAssetError(ReticulatorException):
     """
     Called when a path is not unique
     """
-#endregion
-# region notify classes
-
-# TODO: Replace these with a hash-based edit-detection method?
 
 class NotifyDict(dict):
     """
@@ -71,10 +90,7 @@ class NotifyDict(dict):
 
         if len(args) > 0:
             for key, value in args[0].items():
-                if isinstance(value, dict):
-                    args[0][key] = NotifyDict(value, owner=self.__owner)
-                if isinstance(value, list):
-                    args[0][key] = NotifyList(value, owner=self.__owner)
+                args[0][key] = convert_to_notify_structure(value, self.__owner)
         super().__init__(*args, **kwargs)
 
 
@@ -90,10 +106,7 @@ class NotifyDict(dict):
         return super().__delitem__(v)
 
     def __setitem__(self, attr, value):
-        if isinstance(value, dict):
-            value = NotifyDict(value, owner=self.__owner)
-        if isinstance(value, list):
-            value = NotifyList(value, owner=self.__owner)
+        value = convert_to_notify_structure(value, self.__owner)
 
         if self.get_item(attr) != value:
             if(self.__owner != None):
@@ -111,10 +124,7 @@ class NotifyList(list):
 
         if len(args) > 0:
             for i in range(len(args[0])):
-                if isinstance(args[0][i], dict):
-                    args[0][i] = NotifyDict(args[0][i], owner=self.__owner)
-                if isinstance(args[0][i], list):
-                    args[0][i] = NotifyList(args[0][i], owner=self.__owner)
+                args[0][i] = convert_to_notify_structure(args[0][i], self.__owner)
 
         super().__init__(*args, **kwargs)
 
@@ -123,12 +133,24 @@ class NotifyList(list):
             return self.__getitem__(attr)
         except:
             return None
+    
+    def __delitem__(self, v) -> None:
+        if(self.__owner != None):
+            self.__owner.dirty = True
+        return super().__delitem__(v)
+    
+    def append(self, v):
+        if(self.__owner != None):
+            self.__owner.dirty = True
+        super().append(v)
+
+    def extend(self, v):
+        if(self.__owner != None):
+            self.__owner.dirty = True
+        super().extend(v)
 
     def __setitem__(self, attr, value):
-        if isinstance(value, dict):
-            value = NotifyDict(value, owner=self.__owner)
-        if isinstance(value, list):
-            value = NotifyList(value, owner=self.__owner)
+        value = convert_to_notify_structure(value, self.__owner)
 
         if self.__getitem__(attr) != value:
             if(self.__owner != None):
@@ -136,10 +158,6 @@ class NotifyList(list):
         
         super().__setitem__(attr, value)
 
-# endregion
-
-NO_ARGUMENT = object()
-
 class Resource():
     """
     The top resource in the inheritance chain.
@@ -154,7 +172,7 @@ class Resource():
 
     def __init__(self, file: FileResource = None, pack: Pack = None) -> None:
         # Public
-        self.pack = pack
+        self.pack: Union[ResourcePack, BehaviorPack] = pack
         self.file = file
         self._dirty = False
 
@@ -256,7 +274,7 @@ class FileResource(Resource):
     A resource, which is also a file.
     Contains:
      - File path
-     - ability to mark for deletion
+     - Ability to mark for deletion
     """
     def __init__(self, file_path: str = None, pack: Pack = None) -> None:
         # Initialize resource
@@ -312,7 +330,7 @@ class JsonResource(Resource):
     """
     Parent class, which is responsible for all resources which contain
     json data.
-    Should not be used directly. Use should JsonFileResource, or JsonSubResource.
+    Should not be used directly. Use JsonFileResource, or JsonSubResource instead.
     Contains:
      - Data object
      - Method for interacting with the data
@@ -320,7 +338,7 @@ class JsonResource(Resource):
 
     def __init__(self, data: dict = None, file: FileResource = None, pack: Pack = None) -> None:
         super().__init__(file=file, pack=pack)
-        self.data = self.convert_to_notify(data)
+        self.data = convert_to_notify_structure(data, self)
 
     def _save(self):
         raise NotImplementedError("This json resource cannot be saved.")
@@ -328,19 +346,9 @@ class JsonResource(Resource):
     def _delete(self):
         raise NotImplementedError("This json resource cannot be deleted.")
 
-    def convert_to_notify(self, raw_data):
-        if isinstance(raw_data, dict):
-            return NotifyDict(raw_data, owner=self)
-
-        if isinstance(raw_data, list):
-            return NotifyList(raw_data, owner=self)
-
-        return raw_data
-
     def __str__(self):
         return json.dumps(self.data, indent=2, ensure_ascii=False)
 
-
     def jsonpath_exists(self, json_path:str) -> bool:
         """
         Checks if a jsonpath exists
@@ -355,7 +363,7 @@ class JsonResource(Resource):
         """
         Removes value at jsonpath location.
         """
-        path_exists = self.jsonpath_exists
+        path_exists = self.jsonpath_exists(json_path)
         if path_exists:
             dpath.util.delete(self.data, json_path)
         elif ensure_exists:
@@ -447,9 +455,9 @@ class JsonFileResource(FileResource, JsonResource):
         # resource. This allows assets to be created from scratch, whilst
         # still having an associated file location.
         if data is not None:
-            self.data = NotifyDict(data, owner=self)
+            self.data = convert_to_notify_structure(data, self)
         else:
-            self.data = NotifyDict(self.pack.load_json(self.file_path), owner=self)
+            self.data = convert_to_notify_structure(self.pack.load_json(self.file_path), self)
 
         # Init json resource, which relies on the new data attribute
         JsonResource.__init__(self, data=self.data, file=self, pack=pack)
@@ -469,25 +477,47 @@ class JsonSubResource(JsonResource):
     """
     def __init__(self, parent: Resource = None, json_path: str = None, data: dict = None) -> None:
         super().__init__(data = data, pack = parent.pack, file = parent.file)
+
+        # The parent of the sub-resource is the asset of type Resource
+        # which owns this sub-resource. For example a Component is owned by
+        # either an EntityFileBP, or a ComponentGroup.
         self.parent = parent
+
+        # The jsonpath is the location within the parent resource, where
+        # this sub-resource is stored.
         self.json_path = json_path
-        self.id = self.get_id_from_jsonpath(json_path)
-        self.data = self.convert_to_notify(data)
+
+        # The ID is the "leaf name" of the subresources jsonpath.
+        # For example a 'minecraft:scale' component has an ID of 
+        # 'minecraft:scale', and a data of {'value': 1.0}
+        self._id = self.get_id_from_jsonpath(json_path)
+
+        # The data is the actual data of the sub-resource, minus the id.
+        # For example a 'minecraft:scale' component has a data of {'value': 1.0}
+        self.data = convert_to_notify_structure(data, self)
+
+        # Internal list of resources that are children of this sub-resource.
         self._resources: JsonSubResource = []
+
+        # Register self into parent, so that it can be found by the parent
+        # during saving, etc.
         self.parent.register_resource(self)
     
+    @property
+    def id(self):
+        return self._id
+
+    @id.setter
+    def id(self, id):
+        self.dirty = True
+        self._id = id
+
     # TODO This feels wrong?
     def get_id_from_jsonpath(self, json_path):
         return json_path.split("/")[-1]
 
-    def convert_to_notify(self, raw_data):
-        if isinstance(raw_data, dict):
-            return NotifyDict(raw_data, owner=self)
-
-        if isinstance(raw_data, list):
-            return NotifyList(raw_data, owner=self)
-
-        return raw_data
+    def __repr__(self):
+        return f"{self.__class__.__name__}: {self.id}"
 
     def __str__(self):
         """
@@ -539,6 +569,35 @@ class JsonSubResource(JsonResource):
         self.parent.dirty = True
         self.parent.delete_jsonpath(self.json_path)
 
+class ShortnameResourceTriple(Resource):
+    """
+    A str:Resource pair, which is used to store assets by their shortname.
+
+    Useful in cases such as EntityRP, where a shortname resource is paired
+    with an identifier, which can be converted into a resource
+    """
+    def __init__(self, shortname: str, id: str, resource: Resource, file: FileResource = None, pack: Pack = None) -> None:
+        super().__init__(file=FileResource, pack=pack)
+
+        # The key which is used to identify the resource within this context. 
+        # For example 'move' is the shortname for  the 'animation.dog.move' resource.
+        self.shortname: str = shortname
+
+        # The ID is the identifier for the resource, which is used to get the resource
+        # from the pack. For example 'animation.dog.move' is the id for the 'move' resource.
+        self.id: str = id
+
+        # The resource is the actual resource, which is paired with the shortname.
+        # For example 'animation.dog.move' will be turned into an instance of AnimationRP.
+        self.resource: Resource = resource
+
+    def exists(self):
+        """
+        Whether the underlying resource exists.
+        Useful for detecting cases where a resource is used but missing.
+        """
+        return self.resource is not None
+
 @dataclass
 class Translation:
     """
@@ -549,29 +608,92 @@ class Translation:
     value: str
     comment: str
 
+class Command(Resource):
+    """
+    A command is a wrapper around a string, which represents a single command
+    in a function file.
+
+    To use this class, you can access the 'data' property, and treat it like
+    a string.
+    """
+    def __init__(self, command: str, file: FileResource = None, pack: Pack = None) -> None:
+        super().__init__(file=file, pack=pack)
+
+        # Register self into file, for purpose of saving
+        if file:
+            self.file.register_resource(self)
+
+        # The 'data' is the actual command, which is stored as a string.
+        self._data: str = command
+
+    @property
+    def data(self):
+        return self._data
+    
+    @data.setter
+    def data(self, data):
+        self._data = data
+        self.dirty = True
+
+    @property
+    def dirty(self):
+        return self._dirty
+        
+    @dirty.setter
+    def dirty(self, dirty):
+        """
+        When a command is marked as dirty, it must propagate this to the
+        file, so that the file can be marked as dirty.
+        """
+        self._dirty = dirty
+        self.file.dirty = dirty
+
+    def is_comment(self):
+        return self.data.startswith("#")
+
+    def __str__(self):
+        return self.data
+
+    def __repr__(self):
+        return f"Function: '{self.data}'"
+
 class FunctionFile(FileResource):
     """
     A FunctionFile is a function file, such as run.mcfunction, and is
     made up of many commands.
     """
+
     def __init__(self, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(file_path=file_path, pack=pack)
-        self.__commands : list[str] = []
+        self.__commands : list[Command] = []
     
+    def strip_comments(self):
+        """
+        Strips all comments from the function file.
+        Generally should be used before accessing and using the `commands` property.
+        """
+        self.commands = [c for c in self.commands if not c.is_comment()]
+
     @cached_property
-    def commands(self) -> list[str]:
+    def commands(self) -> list[Command]:
+        """
+        The list of commands in this function file. Every line represents a Command.
+        """
         with open(os.path.join(self.pack.input_path, self.file_path), "r", encoding='utf-8') as function_file:
             for line in function_file.readlines():
-                if line.strip() and not line.startswith("#"):
-                    self.__commands.append(line)
+                command = line.strip()
+                if command:
+                    self.__commands.append(Command(command, file=self, pack=self.pack))
+        self.__commands = NotifyList(self.__commands, owner=self)
         return self.__commands
-
+    
     def _save(self):
         path = os.path.join(self.pack.output_path, self.file_path)
         create_nested_directory(path)
         with open(path, 'w', encoding='utf-8') as file:
+            print(f"SAVING {self.file_name} with {len(self.commands)} commands.")
             for command in self.commands:
-                file.write(command)
+                file.write(command.data + '\n')
 
 class LanguageFile(FileResource):
     """
@@ -803,9 +925,6 @@ class Project():
         self.__behavior_pack.save(force=force)
         self.__resource_pack.save(force=force)
 
-# Forward Declares, to keep pylint from shouting.
-class BehaviorPack(Pack): pass
-class ResourcePack(Pack): pass
 class ResourcePack(Pack):
     def __init__(self, input_path: str, project: Project = None):
         super().__init__(input_path, project=project)
@@ -892,6 +1011,14 @@ class ResourcePack(Pack):
         return self.__items
 
     
+    @cached_property
+    def animations(self) -> list[AnimationRP]:
+        children = []
+        for file in self.animation_files:
+            for child in file.animations:
+                children.append(child)
+        return children
+
     @cached_property
     def render_controllers(self) -> list[RenderControllerRP]:
         children = []
@@ -919,65 +1046,72 @@ class ResourcePack(Pack):
     
     def get_particle(self, identifier:str) -> ParticleFileRP:
         for child in self.particles:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_attachable(self, identifier:str) -> AttachableFileRP:
         for child in self.attachables:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_animation_controller_file(self, file_name:str) -> AnimationControllerFileRP:
         for child in self.animation_controller_files:
-            if child.file_name == file_name:
+            if smart_compare(child.file_name, file_name):
                 return child
         raise AssetNotFoundError(file_name)
 
     def get_animation_file(self, file_name:str) -> AnimationFileRP:
         for child in self.animation_files:
-            if child.file_name == file_name:
+            if smart_compare(child.file_name, file_name):
                 return child
         raise AssetNotFoundError(file_name)
 
     def get_entity(self, identifier:str) -> EntityFileRP:
         for child in self.entities:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_model_file(self, file_name:str) -> ModelFileRP:
         for child in self.model_files:
-            if child.file_name == file_name:
+            if smart_compare(child.file_name, file_name):
                 return child
         raise AssetNotFoundError(file_name)
 
     def get_render_controller_file(self, file_name:str) -> RenderControllerFileRP:
         for child in self.render_controller_files:
-            if child.file_name == file_name:
+            if smart_compare(child.file_name, file_name):
                 return child
         raise AssetNotFoundError(file_name)
 
     
+    def get_animation(self, id:str) -> AnimationRP:
+        for file_child in self.animation_files:
+            for child in file_child.animations:
+                if smart_compare(child.id, id):
+                    return child
+        raise AssetNotFoundError(id)
+
     def get_render_controller(self, id:str) -> RenderControllerRP:
         for file_child in self.render_controller_files:
             for child in file_child.render_controllers:
-                if child.id == id:
+                if smart_compare(child.id, id):
                     return child
         raise AssetNotFoundError(id)
 
     def get_animation_controller(self, id:str) -> AnimationControllerRP:
         for file_child in self.animation_controller_files:
             for child in file_child.animation_controllers:
-                if child.id == id:
+                if smart_compare(child.id, id):
                     return child
         raise AssetNotFoundError(id)
 
     def get_model(self, identifier:str) -> Model:
         for file_child in self.model_files:
             for child in file_child.models:
-                if child.identifier == identifier:
+                if smart_compare(child.identifier, identifier):
                     return child
         raise AssetNotFoundError(identifier)
 
@@ -1007,11 +1141,11 @@ class BehaviorPack(Pack):
         return self.__functions
 
     @cached_property
-    def features_file(self) -> list[FeaturesFileBP]:
+    def features_file(self) -> list[FeatureFileBP]:
         base_directory = os.path.join(self.input_path, "features")
         for local_path in glob.glob(base_directory + "/**/*.json", recursive=True):
             local_path = os.path.relpath(local_path, self.input_path)
-            self.__features_file.append(FeaturesFileBP(file_path = local_path, pack = self))
+            self.__features_file.append(FeatureFileBP(file_path = local_path, pack = self))
             
         return self.__features_file
 
@@ -1091,48 +1225,57 @@ class BehaviorPack(Pack):
     
     def get_function(self, file_path:str) -> FunctionFile:
         for child in self.functions:
-            if child.file_path == file_path:
+            if smart_compare(child.file_path, file_path):
                 return child
         raise AssetNotFoundError(file_path)
 
     def get_feature_rules_file(self, identifier:str) -> FeatureRulesFileBP:
         for child in self.feature_rules_files:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_spawn_rule(self, identifier:str) -> SpawnRuleFile:
         for child in self.spawn_rules:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_recipe(self, identifier:str) -> RecipeFile:
         for child in self.recipes:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_entity(self, identifier:str) -> EntityFileBP:
         for child in self.entities:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_item(self, identifier:str) -> ItemFileBP:
         for child in self.items:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     def get_block(self, identifier:str) -> BlockFileBP:
         for child in self.blocks:
-            if child.identifier == identifier:
+            if smart_compare(child.identifier, identifier):
                 return child
         raise AssetNotFoundError(identifier)
 
     
     
+class FeatureFileBP(JsonFileResource):
+    def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
+        super().__init__(data = data, file_path = file_path, pack = pack)
+        
+    
+    
+    
+    
+    
 class RecipeFile(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
@@ -1189,13 +1332,13 @@ class ParticleFileRP(JsonFileResource):
     
     def get_component(self, id:str) -> JsonSubResource:
         for child in self.components:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
 
     def get_event(self, id:str) -> JsonSubResource:
         for child in self.events:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
 
@@ -1275,7 +1418,7 @@ class RenderControllerFileRP(JsonFileResource):
     
     def get_render_controller(self, id:str) -> RenderControllerRP:
         for child in self.render_controllers:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
 
@@ -1305,7 +1448,7 @@ class AnimationControllerFileRP(JsonFileResource):
     
     def get_animation_controller(self, id:str) -> AnimationControllerRP:
         for child in self.animation_controllers:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
 
@@ -1314,8 +1457,7 @@ class AnimationControllerFileRP(JsonFileResource):
 class SpawnRuleFile(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
-        
-    
+
     @property
     def identifier(self):
         return self.get_jsonpath("minecraft:spawn_rules/description/identifier")
@@ -1327,37 +1469,27 @@ class SpawnRuleFile(JsonFileResource):
     @property
     def format_version(self):
         return self.get_jsonpath("format_version")
-    
+
     @format_version.setter
     def format_version(self, format_version):
         return self.set_jsonpath("format_version", format_version)
 
-    
-    
-    
-    
 class LootTableFile(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
         self.__pools = []
-        
-    
-    
+
     @cached_property
     def pools(self) -> list[LootTablePool]:
         for path, data in self.get_data_at("pools/*"):
             self.__pools.append(LootTablePool(parent = self, json_path = path, data = data))
         return self.__pools
-    
-    
-    
-    
+
 class ItemFileRP(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
         self.__components = []
-        
-    
+
     @property
     def identifier(self):
         return self.get_jsonpath("minecraft:item/description/identifier")
@@ -1380,16 +1512,12 @@ class ItemFileRP(JsonFileResource):
         for path, data in self.get_data_at("minecraft:item/components/*"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
-    
-    
-    
-    
+
 class ItemFileBP(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
         self.__components = []
-        
-    
+
     @property
     def identifier(self):
         return self.get_jsonpath("minecraft:item/description/identifier")
@@ -1398,22 +1526,17 @@ class ItemFileBP(JsonFileResource):
     def identifier(self, identifier):
         return self.set_jsonpath("minecraft:item/description/identifier", identifier)
 
-    
     @cached_property
     def components(self) -> list[Component]:
         for path, data in self.get_data_at("minecraft:item/components"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
-    
-    
-    
-    
+
 class BlockFileBP(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
         self.__components = []
-        
-    
+
     @property
     def identifier(self):
         return self.get_jsonpath("minecraft:block/description/identifier")
@@ -1422,46 +1545,89 @@ class BlockFileBP(JsonFileResource):
     def identifier(self, identifier):
         return self.set_jsonpath("minecraft:block/description/identifier", identifier)
 
-    
     @cached_property
     def components(self) -> list[Component]:
         for path, data in self.get_data_at("minecraft:block/components"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
-    
-    
-    
-    
+
 class EntityFileRP(JsonFileResource):
+    """
+    EntityFileRP is a class which represents a resource pack's entity file.
+    """
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
-        self.__animations = []
-        
-    
+        self.__animations: ShortnameResourceTriple = []
+        self.__models: ShortnameResourceTriple = []
+
     @property
-    def identifier(self):
+    def identifier(self) -> str:
         return self.get_jsonpath("minecraft:client_entity/description/identifier")
-    
+
     @identifier.setter
     def identifier(self, identifier):
         return self.set_jsonpath("minecraft:client_entity/description/identifier", identifier)
 
-    
+    @property
+    def counterpart(self) -> EntityFileBP:
+        try:
+            return self.pack.project.behavior_pack.get_entity(self.identifier)
+        except ReticulatorException:
+            return None
+
     @cached_property
-    def animations(self) -> list[AnimationRP]:
-        for path, data in self.get_data_at("minecraft:client_entity/description/animations/*"):
-            self.__animations.append(AnimationRP(parent = self, json_path = path, data = data))
+    def animations(self) -> list[ShortnameResourceTriple]:
+        """
+        The animations of the entity, as a list of ShortnameResourceTriple objects.
+        Using this structure, you can get the animation's shortname, identifier, and 
+        Resource definition.
+        """
+        data = self.get_jsonpath("minecraft:client_entity/description/animations")
+        for shortname, animation_id in data.items():
+            try:
+                animation = self.pack.get_animation(animation_id)
+            except AssetNotFoundError:
+                animation = None
+
+            self.__animations.append(
+                ShortnameResourceTriple(
+                    shortname,
+                    animation_id,
+                    animation
+                )
+            )
+
         return self.__animations
-    
-    
-    
-    
+
+    @cached_property
+    def models(self) -> list[ShortnameResourceTriple]:
+        data = self.get_jsonpath("minecraft:client_entity/description/geometry")
+        for shortname, geometry_id in data.items():
+            try:
+                geometry = self.pack.get_model(geometry_id)
+            except AssetNotFoundError:
+                geometry = None
+
+            self.__models.append(
+                ShortnameResourceTriple(
+                    shortname,
+                    geometry_id,
+                    geometry
+                )
+            )
+
+        return self.__models
+
 class AnimationFileRP(JsonFileResource):
+    """
+    AnimationFileRP is a class which represents a resource pack's animation file.
+    Since many animations are defined in the same file, it is often more useful.
+    to use the AnimationRP class instead.
+    """
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
-        self.__animations = []
-        
-    
+        self.__animations: AnimationRP = []
+
     @property
     def format_version(self):
         return self.get_jsonpath("format_version")
@@ -1470,16 +1636,12 @@ class AnimationFileRP(JsonFileResource):
     def format_version(self, format_version):
         return self.set_jsonpath("format_version", format_version)
 
-    
     @cached_property
     def animations(self) -> list[AnimationRP]:
         for path, data in self.get_data_at("animations/*"):
             self.__animations.append(AnimationRP(parent = self, json_path = path, data = data))
         return self.__animations
-    
-    
-    
-    
+
 class EntityFileBP(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
@@ -1526,13 +1688,13 @@ class EntityFileBP(JsonFileResource):
     
     def get_component_group(self, id:str) -> ComponentGroup:
         for child in self.component_groups:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
 
     def get_component(self, id:str) -> Component:
         for child in self.components:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
 
@@ -1631,11 +1793,9 @@ class AnimationControllerRP(JsonSubResource):
     
     def get_state(self, id:str) -> AnimationControllerStateRP:
         for child in self.states:
-            if child.id == id:
+            if smart_compare(child.id, id):
                 return child
         raise AssetNotFoundError(id)
-
-    
     
 class Model(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
@@ -1755,9 +1915,4 @@ class Bone(JsonSubResource):
     def cubes(self) -> list[Cube]:
         for path, data in self.get_data_at("cubes/*"):
             self.__cubes.append(Cube(parent = self, json_path = path, data = data))
-        return self.__cubes
-    
-    
-    
-    
-    
\ No newline at end of file
+        return self.__cubes
\ No newline at end of file
